@@ -16,6 +16,8 @@
 #include <ctype.h>
 
 #include <stdlib.h>
+#include <sys/ioctl.h>
+#include <unistd.h>
 
 #ifdef REM_USE_WCHAR
 #include <wctype.h>
@@ -31,10 +33,11 @@
 /* Data structures used by the calendar */
 typedef struct cal_entry {
     struct cal_entry *next;
-    char const *text;
+    char *text;
+    char *raw_text;
     char const *pos;
 #ifdef REM_USE_WCHAR
-    wchar_t const *wc_text;
+    wchar_t *wc_text;
     wchar_t const *wc_pos;
 #endif
     int is_color;
@@ -44,8 +47,12 @@ typedef struct cal_entry {
     DynamicBuffer tags;
     char passthru[PASSTHRU_LEN+1];
     int duration;
-    char const *filename;
+    char *filename;
     int lineno;
+    Trigger trig;
+    TimeTrig tt;
+    int nonconst_expr;
+    int if_depth;
 } CalEntry;
 
 /* Line-drawing sequences */
@@ -132,13 +139,92 @@ static char *VT100Colors[2][2][2][2] /* [Br][R][G][B] */ = {
 static struct line_drawing *linestruct;
 #define DRAW(x) fputs(linestruct->x, stdout)
 
+struct xterm256_colors {
+    int r;
+    int g;
+    int b;
+};
+
+/* Xterm support 256 different colors, numbered from 0 to 255.  The following
+   table is a list of the [r, g, b] components of each Xterm color.  The
+   data was obtained from https://jonasjacek.github.io/colors/ */
+static struct xterm256_colors XTerm256Colors[] =
+{
+    {  0,   0,   0}, {128,   0,   0}, {  0, 128,   0}, {128, 128,   0},
+    {  0,   0, 128}, {128,   0, 128}, {  0, 128, 128}, {192, 192, 192},
+    {128, 128, 128}, {255,   0,   0}, {  0, 255,   0}, {255, 255,   0},
+    {  0,   0, 255}, {255,   0, 255}, {  0, 255, 255}, {255, 255, 255},
+    {  0,   0,   0}, {  0,   0,  95}, {  0,   0, 135}, {  0,   0, 175},
+    {  0,   0, 215}, {  0,   0, 255}, {  0,  95,   0}, {  0,  95,  95},
+    {  0,  95, 135}, {  0,  95, 175}, {  0,  95, 215}, {  0,  95, 255},
+    {  0, 135,   0}, {  0, 135,  95}, {  0, 135, 135}, {  0, 135, 175},
+    {  0, 135, 215}, {  0, 135, 255}, {  0, 175,   0}, {  0, 175,  95},
+    {  0, 175, 135}, {  0, 175, 175}, {  0, 175, 215}, {  0, 175, 255},
+    {  0, 215,   0}, {  0, 215,  95}, {  0, 215, 135}, {  0, 215, 175},
+    {  0, 215, 215}, {  0, 215, 255}, {  0, 255,   0}, {  0, 255,  95},
+    {  0, 255, 135}, {  0, 255, 175}, {  0, 255, 215}, {  0, 255, 255},
+    { 95,   0,   0}, { 95,   0,  95}, { 95,   0, 135}, { 95,   0, 175},
+    { 95,   0, 215}, { 95,   0, 255}, { 95,  95,   0}, { 95,  95,  95},
+    { 95,  95, 135}, { 95,  95, 175}, { 95,  95, 215}, { 95,  95, 255},
+    { 95, 135,   0}, { 95, 135,  95}, { 95, 135, 135}, { 95, 135, 175},
+    { 95, 135, 215}, { 95, 135, 255}, { 95, 175,   0}, { 95, 175,  95},
+    { 95, 175, 135}, { 95, 175, 175}, { 95, 175, 215}, { 95, 175, 255},
+    { 95, 215,   0}, { 95, 215,  95}, { 95, 215, 135}, { 95, 215, 175},
+    { 95, 215, 215}, { 95, 215, 255}, { 95, 255,   0}, { 95, 255,  95},
+    { 95, 255, 135}, { 95, 255, 175}, { 95, 255, 215}, { 95, 255, 255},
+    {135,   0,   0}, {135,   0,  95}, {135,   0, 135}, {135,   0, 175},
+    {135,   0, 215}, {135,   0, 255}, {135,  95,   0}, {135,  95,  95},
+    {135,  95, 135}, {135,  95, 175}, {135,  95, 215}, {135,  95, 255},
+    {135, 135,   0}, {135, 135,  95}, {135, 135, 135}, {135, 135, 175},
+    {135, 135, 215}, {135, 135, 255}, {135, 175,   0}, {135, 175,  95},
+    {135, 175, 135}, {135, 175, 175}, {135, 175, 215}, {135, 175, 255},
+    {135, 215,   0}, {135, 215,  95}, {135, 215, 135}, {135, 215, 175},
+    {135, 215, 215}, {135, 215, 255}, {135, 255,   0}, {135, 255,  95},
+    {135, 255, 135}, {135, 255, 175}, {135, 255, 215}, {135, 255, 255},
+    {175,   0,   0}, {175,   0,  95}, {175,   0, 135}, {175,   0, 175},
+    {175,   0, 215}, {175,   0, 255}, {175,  95,   0}, {175,  95,  95},
+    {175,  95, 135}, {175,  95, 175}, {175,  95, 215}, {175,  95, 255},
+    {175, 135,   0}, {175, 135,  95}, {175, 135, 135}, {175, 135, 175},
+    {175, 135, 215}, {175, 135, 255}, {175, 175,   0}, {175, 175,  95},
+    {175, 175, 135}, {175, 175, 175}, {175, 175, 215}, {175, 175, 255},
+    {175, 215,   0}, {175, 215,  95}, {175, 215, 135}, {175, 215, 175},
+    {175, 215, 215}, {175, 215, 255}, {175, 255,   0}, {175, 255,  95},
+    {175, 255, 135}, {175, 255, 175}, {175, 255, 215}, {175, 255, 255},
+    {215,   0,   0}, {215,   0,  95}, {215,   0, 135}, {215,   0, 175},
+    {215,   0, 215}, {215,   0, 255}, {215,  95,   0}, {215,  95,  95},
+    {215,  95, 135}, {215,  95, 175}, {215,  95, 215}, {215,  95, 255},
+    {215, 135,   0}, {215, 135,  95}, {215, 135, 135}, {215, 135, 175},
+    {215, 135, 215}, {215, 135, 255}, {215, 175,   0}, {215, 175,  95},
+    {215, 175, 135}, {215, 175, 175}, {215, 175, 215}, {215, 175, 255},
+    {215, 215,   0}, {215, 215,  95}, {215, 215, 135}, {215, 215, 175},
+    {215, 215, 215}, {215, 215, 255}, {215, 255,   0}, {215, 255,  95},
+    {215, 255, 135}, {215, 255, 175}, {215, 255, 215}, {215, 255, 255},
+    {255,   0,   0}, {255,   0,  95}, {255,   0, 135}, {255,   0, 175},
+    {255,   0, 215}, {255,   0, 255}, {255,  95,   0}, {255,  95,  95},
+    {255,  95, 135}, {255,  95, 175}, {255,  95, 215}, {255,  95, 255},
+    {255, 135,   0}, {255, 135,  95}, {255, 135, 135}, {255, 135, 175},
+    {255, 135, 215}, {255, 135, 255}, {255, 175,   0}, {255, 175,  95},
+    {255, 175, 135}, {255, 175, 175}, {255, 175, 215}, {255, 175, 255},
+    {255, 215,   0}, {255, 215,  95}, {255, 215, 135}, {255, 215, 175},
+    {255, 215, 215}, {255, 215, 255}, {255, 255,   0}, {255, 255,  95},
+    {255, 255, 135}, {255, 255, 175}, {255, 255, 215}, {255, 255, 255},
+    {  8,   8,   8}, { 18,  18,  18}, { 28,  28,  28}, { 38,  38,  38},
+    { 48,  48,  48}, { 58,  58,  58}, { 68,  68,  68}, { 78,  78,  78},
+    { 88,  88,  88}, { 98,  98,  98}, {108, 108, 108}, {118, 118, 118},
+    {128, 128, 128}, {138, 138, 138}, {148, 148, 148}, {158, 158, 158},
+    {168, 168, 168}, {178, 178, 178}, {188, 188, 188}, {198, 198, 198},
+    {208, 208, 208}, {218, 218, 218}, {228, 228, 228}, {238, 238, 238}
+};
+
 /* Global variables */
 static CalEntry *CalColumn[7];
 
 static int ColSpaces;
 
-static void Colorize(CalEntry const *e);
-static void Decolorize(void);
+static int DidAMonth;
+static int DidADay;
+
+static void ColorizeEntry(CalEntry const *e);
 static void SortCol (CalEntry **col);
 static void DoCalendarOneWeek (int nleft);
 static void DoCalendarOneMonth (void);
@@ -159,6 +245,76 @@ static void WriteBottomCalLine (void);
 static void WriteIntermediateCalLine (void);
 static void WriteCalDays (void);
 
+static void PrintJSONString(char const *s)
+{
+    while (*s) {
+	switch(*s) {
+	case '\b': printf("\\b"); break;
+	case '\f': printf("\\f"); break;
+	case '\n': printf("\\n"); break;
+	case '\r': printf("\\r"); break;
+	case '\t': printf("\\t"); break;
+	case '"':  printf("\\\""); break;
+	case '\\': printf("\\\\"); break;
+	default: printf("%c", *s);
+	}
+	s++;
+    }
+}
+
+static void PrintJSONKeyPairInt(char const *name, int val)
+{
+    printf("\"");
+    PrintJSONString(name);
+    printf("\":%d, ", val);
+}
+
+static void PrintJSONKeyPairString(char const *name, char const *val)
+{
+    /* If value is blank, skip it! */
+    if (!val || !*val) {
+	return;
+    }
+
+    printf("\"");
+    PrintJSONString(name);
+    printf("\":\"");
+    PrintJSONString(val);
+    printf("\", ");
+}
+
+static void PrintJSONKeyPairDate(char const *name, int jul)
+{
+    int y, m, d;
+    if (jul == NO_DATE) {
+	/* Skip it! */
+	return;
+    }
+    FromJulian(jul, &y, &m, &d);
+    printf("\"");
+    PrintJSONString(name);
+    printf("\":\"%04d-%02d-%02d\", ", y, m+1, d);
+
+}
+
+static void PrintJSONKeyPairDateTime(char const *name, int dt)
+{
+    int y, m, d, h, i, k;
+    if (dt == NO_TIME) {
+	/* Skip it! */
+	return;
+    }
+    i = dt / MINUTES_PER_DAY;
+    FromJulian(i, &y, &m, &d);
+    k = dt % MINUTES_PER_DAY;
+    h = k / 60;
+    i = k % 60;
+    printf("\"");
+    PrintJSONString(name);
+    printf("\":\"%04d-%02d-%02dT%02d:%02d\", ", y, m+1, d, h, i);
+
+}
+
 #ifdef REM_USE_WCHAR
 static void PutWideChar(wchar_t const wc)
 {
@@ -202,26 +358,146 @@ static void goff(void)
   printf("%s", linestruct->graphics_off);
 }
 
-static void Decolorize(void)
+static void
+ClampColor(int *r, int *g, int *b)
 {
-  printf("%s", "\x1B[0m");
+    if (TerminalBackground == TERMINAL_BACKGROUND_UNKNOWN) {
+	/* No special clamping if terminal background is unknown */
+	return;
+    }
+    if (TerminalBackground == TERMINAL_BACKGROUND_DARK) {
+	if (*r <= 64 && *g <= 64 && *b <= 64) {
+	    int max = *r;
+	    double factor;
+	    if (*g > max) max = *g;
+	    if (*b > max) max = *b;
+	    if (max == 0) {
+		*r = 65;
+		*g = 65;
+		*b = 65;
+		return;
+	    }
+	    factor = 65.0 / (double) max;
+	    *r = (int) (factor * (double) *r);
+	    *g = (int) (factor * (double) *g);
+	    *b = (int) (factor * (double) *b);
+	}
+	return;
+    }
+    if (TerminalBackground == TERMINAL_BACKGROUND_LIGHT) {
+	if (*r > 191 && *g > 191 && *b > 191) {
+	    int min = *r;
+	    if (*g < min) min = *g;
+	    if (*b < min) min = *b;
+	    double factor = 192.0 / (double) min;
+	    *r = (int) (factor * (double) *r);
+	    *g = (int) (factor * (double) *g);
+	    *b = (int) (factor * (double) *b);
+	}
+    }
 }
 
-static void Colorize(CalEntry const *e)
+char const *
+Decolorize(int r, int g, int b)
 {
-  int bright = 0;
-  int r, g, b;
-  if (e->r > 128 || e->g > 128 || e->b > 128) {
-    bright = 1;
-  }
-  if (e->r > 64) r = 1;
-  else r = 0;
-  if (e->g > 64) g = 1;
-  else g = 0;
-  if (e->b > 64) b = 1;
-  else b = 0;
+    if (!strcmp(Colorize(r, g, b), "")) {
+	return "";
+    }
+    return "\x1B[0m";
+}
 
-  printf("%s", VT100Colors[bright][r][g][b]);
+static char const *
+Colorize256(int r, int g, int b)
+{
+    static char buf[40];
+    int best = -1;
+    int best_dist = 0;
+    int dist;
+    struct xterm256_colors *cur;
+    size_t i;
+
+    ClampColor(&r, &g, &b);
+    for (i=0; i<(sizeof(XTerm256Colors) / sizeof(XTerm256Colors[0])); i++) {
+	cur = &XTerm256Colors[i];
+	dist = ((r - cur->r) * (r - cur->r)) +
+	    ((b - cur->b) * (b - cur->b)) +
+	    ((g - cur->g) * (g - cur->g));
+	if (best == -1 || dist < best_dist) {
+	    best_dist = dist;
+	    best = (int) i;
+	}
+    }
+    cur = &XTerm256Colors[best];
+    sprintf(buf, "\x1B[38;5;%dm", best);
+    return buf;
+}
+
+static char const *
+ColorizeTrue(int r, int g, int b)
+{
+    static char buf[40];
+    ClampColor(&r, &g, &b);
+    sprintf(buf, "\x1B[38;2;%d;%d;%dm", r, g, b);
+    return buf;
+}
+
+char const *
+Colorize(int r, int g, int b)
+{
+    int bright = 0;
+
+    if (UseTrueColors) {
+	return ColorizeTrue(r, g, b);
+    }
+
+    if (Use256Colors) {
+	return Colorize256(r, g, b);
+    }
+    if (r > 128 || g > 128 || b > 128) {
+	bright = 1;
+    }
+    if (r > 64) r = 1;
+    else r = 0;
+    if (g > 64) g = 1;
+    else g = 0;
+    if (b > 64) b = 1;
+    else b = 0;
+
+    if (TerminalBackground == TERMINAL_BACKGROUND_DARK) {
+	/* Convert black-on-black to grey */
+	if (!r && !g && !b) return VT100Colors[1][0][0][0];
+    }
+    if (TerminalBackground == TERMINAL_BACKGROUND_LIGHT) {
+	/* Convert white-on-white to grey */
+	if (r && g && b) return VT100Colors[1][0][0][0];
+    }
+    return VT100Colors[bright][r][g][b];
+}
+
+static void ColorizeEntry(CalEntry const *e)
+{
+    printf("%s", Colorize(e->r, e->g, e->b));
+}
+
+static int
+ComputeCalWidth(int x)
+{
+    struct winsize w;
+    if (x >= 71) {
+	/* Has been set with -w option */
+	return x;
+    }
+    if (!isatty(STDOUT_FILENO)) {
+	/* Output is not a TTY... assume 80 */
+	return 80;
+    }
+    if (ioctl(STDOUT_FILENO, TIOCGWINSZ, &w) < 0) {
+	return 80;
+    }
+    if (w.ws_col < 71) {
+	return 80;
+    }
+    return w.ws_col;
 }
 
 /***************************************************************/
@@ -244,14 +520,25 @@ void ProduceCalendar(void)
     }
     ShouldCache = 1;
 
+    CalWidth = ComputeCalWidth(CalWidth);
+
     ColSpaces = (CalWidth - 9) / 7;
     CalWidth = 7*ColSpaces + 8;
 
     if (CalMonths) {
 	FromJulian(JulianToday, &y, &m, &d);
 	JulianToday = Julian(y, m, 1);
-	while (CalMonths--)
+	DidAMonth = 0;
+	if (PsCal == PSCAL_LEVEL3) {
+	    printf("[\n");
+	}
+	while (CalMonths--) {
 	    DoCalendarOneMonth();
+	    DidAMonth = 1;
+	}
+	if (PsCal == PSCAL_LEVEL3) {
+	    printf("\n]\n");
+	}
 	return;
     } else {
 	if (MondayFirst) JulianToday -= (JulianToday%7);
@@ -373,30 +660,74 @@ static void DoCalendarOneMonth(void)
 
     if (!DoSimpleCalendar) WriteCalHeader();
 
+    DidADay = 0;
+
     if (PsCal) {
 	FromJulian(JulianToday, &y, &m, &d);
-	printf("%s\n", PSBEGIN);
-	printf("%s %d %d %d %d\n",
-	       MonthName[m], y, DaysInMonth(m, y), (JulianToday+1) % 7,
-	       MondayFirst);
-	printf("%s %s %s %s %s %s %s\n",
-	       DayName[6], DayName[0], DayName[1], DayName[2],
-	       DayName[3], DayName[4], DayName[5]);
+	if (PsCal == PSCAL_LEVEL1) {
+	    printf("%s\n", PSBEGIN);
+	} else if (PsCal == PSCAL_LEVEL2) {
+	    printf("%s\n", PSBEGIN2);
+	} else {
+	    if (DidAMonth) {
+		printf(",\n");
+	    }
+	    printf("{\n");
+	}
+	if (PsCal < PSCAL_LEVEL3) {
+	    printf("%s %d %d %d %d\n",
+		   MonthName[m], y, DaysInMonth(m, y), (JulianToday+1) % 7,
+		   MondayFirst);
+	    printf("%s %s %s %s %s %s %s\n",
+		   DayName[6], DayName[0], DayName[1], DayName[2],
+		   DayName[3], DayName[4], DayName[5]);
+	} else {
+	    PrintJSONKeyPairString("monthname", MonthName[m]);
+	    PrintJSONKeyPairInt("year", y);
+	    PrintJSONKeyPairInt("daysinmonth", DaysInMonth(m, y));
+	    PrintJSONKeyPairInt("firstwkday", (JulianToday+1) % 7);
+	    PrintJSONKeyPairInt("mondayfirst", MondayFirst);
+	    printf("\"daynames\":[\"%s\",\"%s\",\"%s\",\"%s\",\"%s\",\"%s\",\"%s\"],",
+		   DayName[6], DayName[0], DayName[1], DayName[2],
+		   DayName[3], DayName[4], DayName[5]);
+	}
 	mm = m-1;
 	if (mm<0) {
 	    mm = 11; yy = y-1;
 	} else yy=y;
 
-	printf("%s %d\n", MonthName[mm], DaysInMonth(mm,yy));
+	if (PsCal < PSCAL_LEVEL3) {
+	    printf("%s %d\n", MonthName[mm], DaysInMonth(mm,yy));
+	} else {
+	    PrintJSONKeyPairString("prevmonthname", MonthName[mm]);
+	    PrintJSONKeyPairInt("daysinprevmonth", DaysInMonth(mm, yy));
+	    PrintJSONKeyPairInt("prevmonthyear", yy);
+	}
 	mm = m+1;
 	if (mm>11) {
 	    mm = 0; yy = y+1;
 	} else yy=y;
-	printf("%s %d\n", MonthName[mm], DaysInMonth(mm,yy));
+	if (PsCal < PSCAL_LEVEL3) {
+	    printf("%s %d\n", MonthName[mm], DaysInMonth(mm,yy));
+	} else {
+	    PrintJSONKeyPairString("nextmonthname", MonthName[mm]);
+	    PrintJSONKeyPairInt("daysinnextmonth", DaysInMonth(mm, yy));
+	    PrintJSONKeyPairInt("nextmonthyear", yy);
+	    printf("\"entries\":[\n");
+	}
     }
     while (WriteCalendarRow()) continue;
 
-    if (PsCal) printf("%s\n", PSEND);
+    if (PsCal == PSCAL_LEVEL1) {
+	printf("%s\n", PSEND);
+    } else if (PsCal == PSCAL_LEVEL2) {
+	printf("%s\n", PSEND2);
+    } else if (PsCal == PSCAL_LEVEL3){
+	if (DidADay) {
+	    printf("\n");
+	}
+	printf("]\n}");
+    }
     if (!DoSimpleCalendar) WriteCalTrailer();
 }
 
@@ -631,9 +962,10 @@ static int WriteOneColLine(int col)
 	if (!*ws && e->next) {
 	    PrintLeft("", ColSpaces, ' ');
 	    CalColumn[col] = e->next;
-	    free((void *)e->text);
-	    free((void *)e->filename);
-	    if (e->wc_text) free((void *)e->wc_text);
+	    free(e->text);
+	    free(e->raw_text);
+	    free(e->filename);
+	    if (e->wc_text) free(e->wc_text);
 	    free(e);
 	    return 1;
 	}
@@ -647,7 +979,7 @@ static int WriteOneColLine(int col)
 
 	/* Colorize reminder if necessary */
 	if (UseVTColors && e->is_color) {
-	    Colorize(e);
+	    ColorizeEntry(e);
 	}
 
 	/* If we couldn't find a space char, print what we have. */
@@ -669,7 +1001,7 @@ static int WriteOneColLine(int col)
 
 	/* Decolorize reminder if necessary */
 	if (UseVTColors && e->is_color) {
-	    Decolorize();
+	    printf("%s", Decolorize(e->r, e->g, e->b));
 	}
 
 	/* Flesh out the rest of the column */
@@ -681,9 +1013,10 @@ static int WriteOneColLine(int col)
 	/* If done, free memory if no next entry. */
 	if (!*ws && !e->next) {
 	    CalColumn[col] = e->next;
-	    free((void *)e->text);
-	    free((void *)e->filename);
-	    if (e->wc_text) free((void *)e->wc_text);
+	    free(e->text);
+	    free(e->raw_text);
+	    free(e->filename);
+	    if (e->wc_text) free(e->wc_text);
 	    free(e);
 	} else {
 	    e->wc_pos = ws;
@@ -699,11 +1032,12 @@ static int WriteOneColLine(int col)
 	if (!*s && e->next) {
 	    PrintLeft("", ColSpaces, ' ');
 	    CalColumn[col] = e->next;
-	    free((void *)e->text);
-	    free((void *)e->filename);
+	    free(e->text);
+	    free(e->filename);
 #ifdef REM_USE_WCHAR
-	    if (e->wc_text) free((void *)e->wc_text);
+	    if (e->wc_text) free(e->wc_text);
 #endif
+	    free(e->raw_text);
 	    free(e);
 	    return 1;
 	}
@@ -717,7 +1051,7 @@ static int WriteOneColLine(int col)
 
 	/* Colorize reminder if necessary */
 	if (UseVTColors && e->is_color) {
-	    Colorize(e);
+	    ColorizeEntry(e);
 	}
 
 	/* If we couldn't find a space char, print what we have. */
@@ -739,7 +1073,7 @@ static int WriteOneColLine(int col)
 
 	/* Decolorize reminder if necessary */
 	if (UseVTColors && e->is_color) {
-	    Decolorize();
+	    printf("%s", Decolorize(e->r, e->g, e->b));
 	}
 
 	/* Flesh out the rest of the column */
@@ -751,11 +1085,12 @@ static int WriteOneColLine(int col)
 	/* If done, free memory if no next entry. */
 	if (!*s && !e->next) {
 	    CalColumn[col] = e->next;
-	    free((void *)e->text);
-	    free((void *)e->filename);
+	    free(e->text);
+	    free(e->filename);
 #ifdef REM_USE_WCHAR
-	    if (e->wc_text) free((void *)e->wc_text);
+	    if (e->wc_text) free(e->wc_text);
 #endif
+	    free(e->raw_text);
 	    free(e);
 	} else {
 	    e->pos = s;
@@ -781,11 +1116,7 @@ static void GenerateCalEntries(int col)
     Parser p;
 
 /* Do some initialization first... */
-    ClearGlobalOmits();
-    DestroyOmitContexts();
-    DestroyVars(0);
-    NumTriggered = 0;
-    ClearLastTriggers();
+    PerIterationInit();
 
     r=IncludeFile(InitialFile);
     if (r) {
@@ -931,14 +1262,16 @@ static int DoCalRem(ParsePtr p, int col)
     CalEntry *CurCol = CalColumn[col];
     CalEntry *e;
     char const *s, *s2;
-    DynamicBuffer buf, obuf, pre_buf;
+    DynamicBuffer buf, obuf, pre_buf, raw_buf;
     Token tok;
+    int nonconst_expr = 0;
 
     int is_color, col_r, col_g, col_b;
 
     is_color = 0;
     DBufInit(&buf);
     DBufInit(&pre_buf);
+    DBufInit(&raw_buf);
 
     /* Parse the trigger date and time */
     if ( (r=ParseRem(p, &trig, &tim, 1)) ) {
@@ -946,6 +1279,20 @@ static int DoCalRem(ParsePtr p, int col)
 	return r;
     }
 
+    if (trig.typ == MSG_TYPE ||
+	trig.typ == CAL_TYPE ||
+	trig.typ == MSF_TYPE) {
+	is_color = (
+	    DefaultColorR != -1
+	    && DefaultColorG != -1
+	    && DefaultColorB != -1);
+	if (is_color) {
+	    col_r = DefaultColorR;
+	    col_g = DefaultColorG;
+	    col_b = DefaultColorB;
+	}
+    }
+
 /* Don't include timed reminders in calendar if -a option supplied. */
     if (DontIssueAts && tim.ttime != NO_TIME) {
 	FreeTrig(&trig);
@@ -1007,6 +1354,8 @@ static int DoCalRem(ParsePtr p, int col)
 	}
     }
 
+    /* Save nonconst_expr flag */
+    nonconst_expr = p->nonconst_expr;
     /* Convert PS and PSF to PASSTHRU */
     if (trig.typ == PS_TYPE) {
 	strcpy(trig.passthru, "PostScript");
@@ -1015,13 +1364,27 @@ static int DoCalRem(ParsePtr p, int col)
 	strcpy(trig.passthru, "PSFile");
 	trig.typ = PASSTHRU_TYPE;
     }
+
+    /* If it's a plain reminder but we have a default color, add the
+       three colors to the prebuf and change passthru to "COLOR" */
+    if (trig.typ == MSG_TYPE ||
+	trig.typ == CAL_TYPE ||
+	trig.typ == MSF_TYPE) {
+	if (PsCal && is_color) {
+	    char cbuf[24];
+	    sprintf(cbuf, "%d %d %d ", col_r, col_g, col_b);
+	    DBufPuts(&pre_buf, cbuf);
+	    strcpy(trig.passthru, "COLOR");
+	    /* Don't change trig.typ or next if() will trigger! */
+	}
+    }
     if (trig.typ == PASSTHRU_TYPE) {
-	if (!PsCal && strcmp(trig.passthru, "COLOR") && strcmp(trig.passthru, "COLOUR")) {
+	if (!PsCal && StrCmpi(trig.passthru, "COLOR") && StrCmpi(trig.passthru, "COLOUR")) {
 	    FreeTrig(&trig);
 	    return OK;
 	}
-	if (!strcmp(trig.passthru, "COLOR") ||
-	    !strcmp(trig.passthru, "COLOUR")) {
+	if (!StrCmpi(trig.passthru, "COLOR") ||
+	    !StrCmpi(trig.passthru, "COLOUR")) {
 	    is_color = 1;
 	    /* Strip off the three color numbers */
 	    DBufFree(&buf);
@@ -1070,20 +1433,31 @@ static int DoCalRem(ParsePtr p, int col)
 	 ShouldTriggerReminder(&trig, &tim, jul, &err))) {
 	NumTriggered++;
 
+	/* The parse_ptr should not be nested, but just in case... */
+	if (!p->isnested) {
+	    if (DBufPuts(&raw_buf, p->pos) != OK) {
+		DBufFree(&obuf);
+		DBufFree(&pre_buf);
+		FreeTrig(&trig);
+		return E_NO_MEM;
+	    }
+	}
 	if (DoSimpleCalendar || tim.ttime != NO_TIME) {
 	    /* Suppress time if it's not today or if it's a non-COLOR special */
 	    if (jul != JulianToday ||
 		(trig.typ == PASSTHRU_TYPE &&
-		 strcmp(trig.passthru, "COLOUR") &&
-		 strcmp(trig.passthru, "COLOR"))) {
+		 StrCmpi(trig.passthru, "COLOUR") &&
+		 StrCmpi(trig.passthru, "COLOR"))) {
 		if (DBufPuts(&obuf, SimpleTime(NO_TIME)) != OK) {
 		    DBufFree(&obuf);
+		    DBufFree(&raw_buf);
 		    DBufFree(&pre_buf);
 		    FreeTrig(&trig);
 		    return E_NO_MEM;
 		}
 	    } else {
 		if (DBufPuts(&obuf, CalendarTime(tim.ttime, tim.duration)) != OK) {
+		    DBufFree(&raw_buf);
 		    DBufFree(&obuf);
 		    DBufFree(&pre_buf);
 		    FreeTrig(&trig);
@@ -1101,6 +1475,7 @@ static int DoCalRem(ParsePtr p, int col)
 		if (!DoCoerce(STR_TYPE, &v)) {
 		    if (DBufPuts(&obuf, v.v.str) != OK) {
 			DestroyValue(v);
+			DBufFree(&raw_buf);
 			DBufFree(&obuf);
 			DBufFree(&pre_buf);
 			FreeTrig(&trig);
@@ -1122,12 +1497,14 @@ static int DoCalRem(ParsePtr p, int col)
 	if (r) {
 	    DBufFree(&pre_buf);
 	    DBufFree(&obuf);
+	    DBufFree(&raw_buf);
 	    FreeTrig(&trig);
 	    return r;
 	}
 	if (DBufLen(&obuf) <= oldLen) {
 	    DBufFree(&obuf);
 	    DBufFree(&pre_buf);
+	    DBufFree(&raw_buf);
 	    FreeTrig(&trig);
 	    return OK;
 	}
@@ -1141,6 +1518,7 @@ static int DoCalRem(ParsePtr p, int col)
 		if (!DoCoerce(STR_TYPE, &v)) {
 		    if (DBufPuts(&obuf, v.v.str) != OK) {
 			DestroyValue(v);
+			DBufFree(&raw_buf);
 			DBufFree(&obuf);
 			DBufFree(&pre_buf);
 			FreeTrig(&trig);
@@ -1157,10 +1535,15 @@ static int DoCalRem(ParsePtr p, int col)
 	e = NEW(CalEntry);
 	if (!e) {
 	    DBufFree(&obuf);
+	    DBufFree(&raw_buf);
 	    DBufFree(&pre_buf);
 	    FreeTrig(&trig);
 	    return E_NO_MEM;
 	}
+	e->nonconst_expr = nonconst_expr;
+	e->if_depth = NumIfs;
+	e->trig = trig;
+	e->tt = tim;
 #ifdef REM_USE_WCHAR
 	e->wc_pos = NULL;
 	e->wc_text = NULL;
@@ -1170,9 +1553,13 @@ static int DoCalRem(ParsePtr p, int col)
 	e->g = col_g;
 	e->b = col_b;
 	e->text = StrDup(s);
+	e->raw_text = StrDup(DBufValue(&raw_buf));
+	DBufFree(&raw_buf);
 	DBufFree(&obuf);
 	DBufFree(&pre_buf);
-	if (!e->text) {
+	if (!e->text || !e->raw_text) {
+	    if (e->text) free(e->text);
+	    if (e->raw_text) free(e->raw_text);
 	    free(e);
 	    FreeTrig(&trig);
 	    return E_NO_MEM;
@@ -1190,12 +1577,14 @@ static int DoCalRem(ParsePtr p, int col)
 	e->priority = trig.priority;
 	e->filename = StrDup(FileName);
 	if(!e->filename) {
+	    if (e->text) free(e->text);
+	    if (e->raw_text) free(e->raw_text);
 	    free(e);
 	    return E_NO_MEM;
 	}
 	e->lineno = LineNo;
 
-	if (trig.typ == PASSTHRU_TYPE) {
+	if (trig.typ == PASSTHRU_TYPE || is_color) {
 	    StrnCpy(e->passthru, trig.passthru, PASSTHRU_LEN);
 	} else {
 	    e->passthru[0] = 0;
@@ -1213,23 +1602,8 @@ static int DoCalRem(ParsePtr p, int col)
     return OK;
 }
 
-/***************************************************************/
-/*                                                             */
-/*  WriteSimpleEntries                                         */
-/*                                                             */
-/*  Write entries in 'simple calendar' format.                 */
-/*                                                             */
-/***************************************************************/
-static void WriteSimpleEntries(int col, int jul)
+static void WriteSimpleEntryProtocol1(CalEntry *e)
 {
-    CalEntry *e = CalColumn[col];
-    CalEntry *n;
-    int y, m, d;
-
-    FromJulian(jul, &y, &m, &d);
-    while(e) {
-	if (DoPrefixLineNo) printf("# fileinfo %d %s\n", e->lineno, e->filename);
-	printf("%04d/%02d/%02d", y, m+1, d);
 	if (e->passthru[0]) {
 	    printf(" %s", e->passthru);
 	} else {
@@ -1251,10 +1625,186 @@ static void WriteSimpleEntries(int col, int jul)
 	    printf("* ");
 	}
 	printf("%s\n", e->text);
-	free((void *)e->text);
-	free((void *)e->filename);
+}
+
+
+static void WriteSimpleEntryProtocol2(CalEntry *e, int today)
+{
+    int done = 0;
+    if (DoPrefixLineNo) {
+	PrintJSONKeyPairString("filename", e->filename);
+	PrintJSONKeyPairInt("lineno", e->lineno);
+    }
+    PrintJSONKeyPairString("passthru", e->passthru);
+    PrintJSONKeyPairString("tags", DBufValue(&(e->tags)));
+    if (e->duration != NO_TIME) {
+	PrintJSONKeyPairInt("duration", e->duration);
+    }
+    if (e->time != NO_TIME) {
+	PrintJSONKeyPairInt("time", e->time);
+	if (e->tt.delta) {
+	    PrintJSONKeyPairInt("tdelta", e->tt.delta);
+	}
+	if (e->tt.rep) {
+	    PrintJSONKeyPairInt("trep", e->tt.rep);
+	}
+    }
+    if (e->trig.eventduration != NO_TIME) {
+	PrintJSONKeyPairInt("eventduration", e->trig.eventduration);
+    }
+    /* wd is an array of days from 0=monday to 6=sunday.
+       We convert to array of strings */
+    if (e->trig.wd != NO_WD) {
+	printf("\"wd\":[");
+	done = 0;
+	int i;
+	for (i=0; i<7; i++) {
+	    if (e->trig.wd & (1 << i)) {
+		if (done) {
+		    printf(",");
+		}
+		done = 1;
+		printf("\"%s\"", EnglishDayName[i]);
+	    }
+	}
+	printf("], ");
+    }
+    if (e->trig.d != NO_DAY) {
+	PrintJSONKeyPairInt("d", e->trig.d);
+    }
+    if (e->trig.m != NO_MON) {
+	PrintJSONKeyPairInt("m", e->trig.m+1);
+    }
+    if (e->trig.y != NO_YR) {
+	PrintJSONKeyPairInt("y", e->trig.y);
+    }
+    PrintJSONKeyPairDateTime("eventstart", e->trig.eventstart);
+    if (e->trig.back) {
+	PrintJSONKeyPairInt("back", e->trig.back);
+    }
+    if (e->trig.delta) {
+	PrintJSONKeyPairInt("delta", e->trig.delta);
+    }
+    if (e->trig.rep) {
+	PrintJSONKeyPairInt("rep", e->trig.rep);
+    }
+    if (e->nonconst_expr) {
+	PrintJSONKeyPairInt("nonconst_expr", e->nonconst_expr);
+    }
+    if (e->if_depth) {
+	PrintJSONKeyPairInt("if_depth", e->if_depth);
+    }
+    switch(e->trig.skip) {
+    case SKIP_SKIP:
+	PrintJSONKeyPairString("skip", "SKIP");
+	break;
+    case BEFORE_SKIP:
+	PrintJSONKeyPairString("skip", "BEFORE");
+	break;
+    case AFTER_SKIP:
+	PrintJSONKeyPairString("skip", "AFTER");
+	break;
+    }
+    /* Local omit is an array of days from 0=monday to 6=sunday.
+       We convert to array of strings */
+    if (e->trig.localomit != NO_WD) {
+	printf("\"localomit\":[");
+	done = 0;
+	int i;
+	for (i=0; i<7; i++) {
+	    if (e->trig.localomit & (1 << i)) {
+		if (done) {
+		    printf(",");
+		}
+		done = 1;
+		printf("\"%s\"", EnglishDayName[i]);
+	    }
+	}
+	printf("], ");
+    }
+    PrintJSONKeyPairDate("until", e->trig.until);
+    if (e->trig.once != NO_ONCE) {
+	PrintJSONKeyPairInt("once", e->trig.once);
+    }
+    if (e->trig.scanfrom != today) {
+	PrintJSONKeyPairDate("scanfrom", e->trig.scanfrom);
+    }
+    PrintJSONKeyPairDate("from", e->trig.from);
+    PrintJSONKeyPairInt("priority", e->trig.priority);
+
+    if (e->is_color) {
+	PrintJSONKeyPairInt("r", e->r);
+	PrintJSONKeyPairInt("g", e->g);
+	PrintJSONKeyPairInt("b", e->b);
+    } else if (!StrCmpi(e->passthru, "SHADE")) {
+	int r, g, b, n;
+	n = sscanf(e->text, "%d %d %d", &r, &g, &b);
+	if (n < 3) {
+	    g = r;
+	    b = r;
+	}
+	if (r < 0) r = 0;
+	else if (r > 255) r = 255;
+	if (g < 0) g = 0;
+	else if (g > 255) g = 255;
+	if (b < 0) b = 0;
+	else if (b > 255) b = 255;
+	PrintJSONKeyPairInt("r", r);
+	PrintJSONKeyPairInt("g", g);
+	PrintJSONKeyPairInt("b", b);
+    }
+    /* Only print rawbody if it differs from body */
+    if (strcmp(e->raw_text, e->text)) {
+	PrintJSONKeyPairString("rawbody", e->raw_text);
+    }
+    printf("\"body\":\"");
+    PrintJSONString(e->text);
+    printf("\"");
+}
+
+/***************************************************************/
+/*                                                             */
+/*  WriteSimpleEntries                                         */
+/*                                                             */
+/*  Write entries in 'simple calendar' format.                 */
+/*                                                             */
+/***************************************************************/
+static void WriteSimpleEntries(int col, int jul)
+{
+    CalEntry *e = CalColumn[col];
+    CalEntry *n;
+    int y, m, d;
+
+    FromJulian(jul, &y, &m, &d);
+    while(e) {
+	if (DoPrefixLineNo) {
+	    if (PsCal != PSCAL_LEVEL2 && PsCal != PSCAL_LEVEL3) {
+		printf("# fileinfo %d %s\n", e->lineno, e->filename);
+	    }
+	}
+	if (PsCal >= PSCAL_LEVEL2) {
+	    if (PsCal == PSCAL_LEVEL3) {
+		if (DidADay) {
+		    printf(",\n");
+		}
+	    }
+	    DidADay = 1;
+	    printf("{\"date\":\"%04d-%02d-%02d\", ", y, m+1, d);
+	    WriteSimpleEntryProtocol2(e, jul);
+	    printf("}");
+	    if (PsCal != PSCAL_LEVEL3) {
+		printf("\n");
+	    }
+	} else {
+	    printf("%04d/%02d/%02d", y, m+1, d);
+	    WriteSimpleEntryProtocol1(e);
+	}
+
+	free(e->text);
+	free(e->raw_text);
+	free(e->filename);
 #ifdef REM_USE_WCHAR
-	if (e->wc_text) free((void *)e->wc_text);
+	if (e->wc_text) free(e->wc_text);
 #endif
 	n = e->next;
 	free(e);
@@ -1531,6 +2081,8 @@ char const *SynthesizeTag(void)
     unsigned char buf[16];
     static char out[128];
     MD5Init(&ctx);
+    MD5Update(&ctx, (unsigned char *) FileName, strlen(FileName));
+    MD5Update(&ctx, (unsigned char *) &LineNo, sizeof(LineNo));
     MD5Update(&ctx, (unsigned char *) CurLine, strlen(CurLine));
     MD5Final(buf, &ctx);
     sprintf(out, "__syn__%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x",
