@@ -43,7 +43,7 @@ ComputeTrigDuration(TimeTrig *t)
 	t->duration == NO_TIME) {
 	return 0;
     }
-    return (t->ttime + t->duration - 1) / 1440;
+    return (t->ttime + t->duration - 1) / MINUTES_PER_DAY;
 }
 
 /***************************************************************/
@@ -389,6 +389,10 @@ int ParseRem(ParsePtr s, Trigger *trig, TimeTrig *tim, int save_in_globals)
 	    r=ParseToken(s, &buf);
 	    if (r) return r;
 	    StrnCpy(trig->omitfunc, DBufValue(&buf), VAR_NAME_LEN);
+
+	    /* An OMITFUNC counts as a nonconst_expr! */
+	    s->expr_happened = 1;
+	    s->nonconst_expr = 1;
 	    DBufFree(&buf);
 	    break;
 
@@ -759,20 +763,24 @@ int TriggerReminder(ParsePtr p, Trigger *t, TimeTrig *tim, int jul)
     char const *s;
     Value v;
 
+    int red = -1, green = -1, blue = -1;
+    int is_color = 0;
+
     DBufInit(&buf);
     DBufInit(&calRow);
     DBufInit(&pre_buf);
     if (t->typ == RUN_TYPE && RunDisabled) return E_RUN_DISABLED;
-    if ((t->typ == PASSTHRU_TYPE && strcmp(t->passthru, "COLOR") && strcmp(t->passthru, "COLOUR")) ||
+    if ((t->typ == PASSTHRU_TYPE && StrCmpi(t->passthru, "COLOR") && StrCmpi(t->passthru, "COLOUR")) ||
 	t->typ == CAL_TYPE ||
 	t->typ == PS_TYPE ||
 	t->typ == PSF_TYPE)
 	return OK;
 
     /* Handle COLOR types */
-    if (t->typ == PASSTHRU_TYPE && (!strcmp(t->passthru, "COLOR") || !strcmp(t->passthru, "COLOUR"))) {
+    if (t->typ == PASSTHRU_TYPE && (!StrCmpi(t->passthru, "COLOR") || !StrCmpi(t->passthru, "COLOUR"))) {
 	/* Strip off three tokens */
 	r = ParseToken(p, &buf);
+	sscanf(DBufValue(&buf), "%d", &red);
 	if (!NextMode) {
 	    DBufPuts(&pre_buf, DBufValue(&buf));
 	    DBufPutc(&pre_buf, ' ');
@@ -780,6 +788,7 @@ int TriggerReminder(ParsePtr p, Trigger *t, TimeTrig *tim, int jul)
 	DBufFree(&buf);
 	if (r) return r;
 	r = ParseToken(p, &buf);
+	sscanf(DBufValue(&buf), "%d", &green);
 	if (!NextMode) {
 	    DBufPuts(&pre_buf, DBufValue(&buf));
 	    DBufPutc(&pre_buf, ' ');
@@ -787,6 +796,7 @@ int TriggerReminder(ParsePtr p, Trigger *t, TimeTrig *tim, int jul)
 	DBufFree(&buf);
 	if (r) return r;
 	r = ParseToken(p, &buf);
+	sscanf(DBufValue(&buf), "%d", &blue);
 	if (!NextMode) {
 	    DBufPuts(&pre_buf, DBufValue(&buf));
 	    DBufPutc(&pre_buf, ' ');
@@ -870,6 +880,23 @@ int TriggerReminder(ParsePtr p, Trigger *t, TimeTrig *tim, int jul)
 	return OK;
     }
 
+    /* Correct colors */
+    if (UseVTColors) {
+	if (red == -1 && green == -1 && blue == -1) {
+	    if (DefaultColorR != -1 && DefaultColorG != -1 && DefaultColorB != -1) {
+		red = DefaultColorR;
+		green = DefaultColorG;
+		blue = DefaultColorB;
+	    }
+	}
+	if (red >= 0 && green >= 0 && blue >= 0) {
+	    is_color = 1;
+	    if (red > 255) red = 255;
+	    if (green > 255) green = 255;
+	    if (blue > 255) blue = 255;
+	}
+    }
+
     /* Put the substituted string into the substitution buffer */
 
     /* Don't use msgprefix() on RUN-type reminders */
@@ -880,6 +907,9 @@ int TriggerReminder(ParsePtr p, Trigger *t, TimeTrig *tim, int jul)
 	    r = EvalExpr(&s, &v, NULL);
 	    if (!r) {
 		if (!DoCoerce(STR_TYPE, &v)) {
+		    if (is_color) {
+			DBufPuts(&buf, Colorize(red, green, blue));
+		    }
 		    if (DBufPuts(&buf, v.v.str) != OK) {
 			DBufFree(&buf);
 			DestroyValue(v);
@@ -891,6 +921,9 @@ int TriggerReminder(ParsePtr p, Trigger *t, TimeTrig *tim, int jul)
 	}
     }
 
+    if (is_color) {
+	DBufPuts(&buf, Colorize(red, green, blue));
+    }
     if ( (r=DoSubst(p, &buf, t, tim, jul, NORMAL_MODE)) ) return r;
     if (t->typ != RUN_TYPE) {
 	if (UserFuncExists("msgsuffix") == 1) {
@@ -899,6 +932,9 @@ int TriggerReminder(ParsePtr p, Trigger *t, TimeTrig *tim, int jul)
 	    r = EvalExpr(&s, &v, NULL);
 	    if (!r) {
 		if (!DoCoerce(STR_TYPE, &v)) {
+		    if (is_color) {
+			DBufPuts(&buf, Colorize(red, green, blue));
+		    }
 		    if (DBufPuts(&buf, v.v.str) != OK) {
 			DBufFree(&buf);
 			DestroyValue(v);
@@ -910,6 +946,10 @@ int TriggerReminder(ParsePtr p, Trigger *t, TimeTrig *tim, int jul)
 	}
     }
 
+    if (is_color) {
+	DBufPuts(&buf, Decolorize(red, green, blue));
+    }
+
     if ((!MsgCommand && t->typ == MSG_TYPE) || t->typ == MSF_TYPE) {
 	if (DBufPutc(&buf, '\n') != OK) {
 	    DBufFree(&buf);
