@@ -656,8 +656,13 @@ int DoSubst(ParsePtr p, DynamicBuffer *dbuf, Trigger *t, TimeTrig *tt, int jul,
 	    break;
 
 	case '"':
-	    if (DBufPutc(dbuf, QUOTE_MARKER) != OK) return E_NO_MEM;
-	    has_quote = 1;
+	    if (PsCal != PSCAL_LEVEL3) {
+		if (DBufPutc(dbuf, QUOTE_MARKER) != OK) return E_NO_MEM;
+		has_quote = 1;
+	    } else {
+		if (DBufPutc(dbuf, '%') != OK) return E_NO_MEM;
+		if (DBufPutc(dbuf, c) != OK) return E_NO_MEM;
+	    }
 	    break;
 
 	default:
