@@ -5,7 +5,7 @@
 /*  This file contains routines to parse and evaluate          */
 /*  expressions.                                               */
 /*                                                             */
-/*  Copyright 1992-2018 by Dianne Skoll                        */
+/*  Copyright 1992-2020 by Dianne Skoll                        */
 /*                                                             */
 /***************************************************************/
 
@@ -30,7 +30,6 @@
 #define LE 4
 #define NE 5
 
-static char CoerceBuf[512];
 extern int NumFuncs;
 
 static int Multiply(void), Divide(void), Mod(void), Add(void),
@@ -578,6 +577,8 @@ int DoCoerce(char type, Value *v)
     int h, d, m, y, i, k;
     char const *s;
 
+    char coerce_buf[128];
+
     /* Do nothing if value is already the right type */
     if (type == v->type) return OK;
 
@@ -605,12 +606,12 @@ int DoCoerce(char type, Value *v)
 	}
     case STR_TYPE:
 	switch(v->type) {
-	case INT_TYPE: sprintf(CoerceBuf, "%d", v->v.val); break;
-	case TIME_TYPE: sprintf(CoerceBuf, "%02d%c%02d", v->v.val / 60,
+	case INT_TYPE: sprintf(coerce_buf, "%d", v->v.val); break;
+	case TIME_TYPE: sprintf(coerce_buf, "%02d%c%02d", v->v.val / 60,
 			       TimeSep, v->v.val % 60);
 	break;
 	case DATE_TYPE: FromJulian(v->v.val, &y, &m, &d);
-	    sprintf(CoerceBuf, "%04d%c%02d%c%02d",
+	    sprintf(coerce_buf, "%04d%c%02d%c%02d",
 		    y, DateSep, m+1, DateSep, d);
 	    break;
 	case DATETIME_TYPE:
@@ -619,13 +620,13 @@ int DoCoerce(char type, Value *v)
 	    k = v->v.val % MINUTES_PER_DAY;
 	    h = k / 60;
 	    i = k % 60;
-	    sprintf(CoerceBuf, "%04d%c%02d%c%02d%c%02d%c%02d",
+	    sprintf(coerce_buf, "%04d%c%02d%c%02d%c%02d%c%02d",
 		    y, DateSep, m+1, DateSep, d, DateTimeSep, h, TimeSep, i);
 	    break;
 	default: return E_CANT_COERCE;
 	}
 	v->type = STR_TYPE;
-	v->v.str = StrDup(CoerceBuf);
+	v->v.str = StrDup(coerce_buf);
 	if (!v->v.str) {
 	    v->type = ERR_TYPE;
 	    return E_NO_MEM;
@@ -765,9 +766,9 @@ static int Add(void)
 	return OK;
     }
 
-/* If it's a datetime plus an int, add 'em */
-    if ((v1.type == DATETIME_TYPE && v2.type == INT_TYPE) ||
-	(v1.type == INT_TYPE && v2.type == DATETIME_TYPE)) {
+/* If it's a datetime plus an int or a time, add 'em */
+    if ((v1.type == DATETIME_TYPE && (v2.type == INT_TYPE || v2.type == TIME_TYPE)) ||
+	((v1.type == INT_TYPE || v1.type == TIME_TYPE) && v2.type == DATETIME_TYPE)) {
 	v1.v.val += v2.v.val;
 	if (v1.v.val < 0) return E_DATE_OVER;
 	v1.type = DATETIME_TYPE;
@@ -775,9 +776,11 @@ static int Add(void)
 	return OK;
     }
 
-/* If it's a time plus an int, add 'em mod MINUTES_PER_DAY */
+/* If it's a time plus an int or a time plus a time,
+   add 'em mod MINUTES_PER_DAY */
     if ((v1.type == TIME_TYPE && v2.type == INT_TYPE) ||
-	(v1.type == INT_TYPE && v2.type == TIME_TYPE)) {
+	(v1.type == INT_TYPE && v2.type == TIME_TYPE) ||
+	(v1.type == TIME_TYPE && v2.type == TIME_TYPE)) {
 	v1.v.val = (v1.v.val + v2.v.val) % MINUTES_PER_DAY;
 	if (v1.v.val < 0) v1.v.val += MINUTES_PER_DAY;
 	v1.type = TIME_TYPE;
@@ -808,7 +811,7 @@ static int Add(void)
 	    return E_NO_MEM;
 	}
 	strcpy(v3.v.str, v1.v.str);
-	strcat(v3.v.str, v2.v.str);
+	strcpy(v3.v.str+l1, v2.v.str);
 	DestroyValue(v1); DestroyValue(v2);
 	PushValStack(v3);
 	return OK;
@@ -851,8 +854,9 @@ static int Subtract(void)
 	return OK;
     }
 
-    /* If it's a datetime minus an int, do subtraction, checking for underflow */
-    if (v1.type == DATETIME_TYPE && v2.type == INT_TYPE) {
+    /* If it's a datetime minus an int or a time, do subtraction,
+     * checking for underflow */
+    if (v1.type == DATETIME_TYPE && (v2.type == INT_TYPE || v2.type == TIME_TYPE)) {
 	v1.v.val -= v2.v.val;
 	if (v1.v.val < 0) return E_DATE_OVER;
 	PushValStack(v1);
