@@ -46,7 +46,7 @@ EXTERN  INIT(   int     DoPrefixLineNo, 0);
 EXTERN	INIT(	int	MondayFirst, 0);
 EXTERN	INIT(	int	Iterations, 1);
 EXTERN  INIT(   int     PsCal, 0);
-EXTERN  INIT(   int     CalWidth, 80);
+EXTERN  INIT(   int     CalWidth, -1);
 EXTERN  INIT(   int     CalWeeks, 0);
 EXTERN  INIT(   int     CalMonths, 0);
 EXTERN  INIT(	int 	Hush, 0);
@@ -73,6 +73,9 @@ EXTERN  INIT(   int     Daemon, 0);
 EXTERN  INIT(   char    DateSep, DATESEP);
 EXTERN  INIT(   char    TimeSep, TIMESEP);
 EXTERN  INIT(   char    DateTimeSep, DATETIMESEP);
+EXTERN  INIT(   int     DefaultColorR, -1);
+EXTERN  INIT(   int     DefaultColorB, -1);
+EXTERN  INIT(   int     DefaultColorG, -1);
 EXTERN  INIT(   int     SynthesizeTags, 0);
 EXTERN  INIT(   int     ScFormat, SC_AMPM);
 EXTERN  INIT(   int     MaxSatIter, 150);
@@ -100,6 +103,9 @@ EXTERN  INIT(   int     CalPad, 1);
 EXTERN  INIT(   int     UseVTChars, 0);
 EXTERN  INIT(   int     UseUTF8Chars, 0);
 EXTERN  INIT(   int     UseVTColors, 0);
+EXTERN  INIT(   int     Use256Colors, 0);
+EXTERN  INIT(   int     UseTrueColors, 0);
+EXTERN  INIT(   int     TerminalBackground, TERMINAL_BACKGROUND_UNKNOWN);
 
 /* Latitude and longitude */
 EXTERN  INIT(	int	  LatDeg, LAT_DEG);
