@@ -38,6 +38,7 @@
  *             simple calendar format.
  *  -r       = Disallow RUN mode
  *  -c[n]    = Produce a calendar for n months (default = 1)
+ *  -@[n,m]  = Colorize n=0 VT100 n=1 85 n=2 True m=0 dark terminal m=1 light
  *  -w[n,n,n] = Specify output device width, padding and spacing
  *  -s[n]    = Produce calendar in "simple calendar" format
  *  -p[n]    = Produce calendar in format compatible with rem2ps
@@ -135,7 +136,7 @@ void InitRemind(int argc, char const *argv[])
     int InvokedAsRem = 0;
     char const *s;
     int weeks;
-
+    int x;
     int jul;
 
 #if defined(__APPLE__)
@@ -190,6 +191,7 @@ void InitRemind(int argc, char const *argv[])
 
     /* Parse the command-line options */
     i = 1;
+
     while (i < argc) {
 	arg = argv[i];
 	if (*arg != '-') break; /* Exit the loop if it's not an option */
@@ -204,6 +206,27 @@ void InitRemind(int argc, char const *argv[])
 	while (*arg) {
 	    switch(*arg++) {
 
+	    case '@':
+		UseVTColors = 1;
+		if (*arg) {
+		    PARSENUM(x, arg);
+		    if (x == 1) {
+			Use256Colors = 1;
+		    } else if (x == 2) {
+			UseTrueColors = 1;
+		    }
+		}
+		if (*arg == ',') {
+		    arg++;
+		    PARSENUM(x, arg);
+		    if (x == 0) {
+			TerminalBackground = TERMINAL_BACKGROUND_DARK;
+		    } else if (x == 1) {
+			TerminalBackground = TERMINAL_BACKGROUND_LIGHT;
+		    }
+		}
+		break;
+
 	    case 'j':
 	    case 'J':
 		PurgeMode = 1;
@@ -400,9 +423,21 @@ void InitRemind(int argc, char const *argv[])
 	    case 'p':
 	    case 'P':
 		DoSimpleCalendar = 1;
-		PsCal = 1;
-		if (*arg == 'a' || *arg == 'A') {
-		    DoSimpleCalDelta = 1;
+		PsCal = PSCAL_LEVEL1;
+		while (*arg == 'a' || *arg == 'A' ||
+		       *arg == 'p' || *arg == 'P') {
+		    if (*arg == 'a' || *arg == 'A') {
+			DoSimpleCalDelta = 1;
+		    } else if (*arg == 'p' || *arg == 'P') {
+			/* JSON interchange formats always include
+			   file and line number info */
+			DoPrefixLineNo = 1;
+			if (PsCal == PSCAL_LEVEL1) {
+			    PsCal = PSCAL_LEVEL2;
+			} else {
+			    PsCal = PSCAL_LEVEL3;
+			}
+		    }
 		    arg++;
 		}
 		PARSENUM(CalMonths, arg);
@@ -418,7 +453,10 @@ void InitRemind(int argc, char const *argv[])
 	    case 'W':
 		if (*arg != ',') {
 		    PARSENUM(CalWidth, arg);
-		    if (CalWidth < 71) CalWidth = 71;
+		    if (CalWidth != 0 && CalWidth < 71) CalWidth = 71;
+		    if (CalWidth == 0) {
+			CalWidth = -1;
+		    }
 		}
 		if (*arg == ',') {
 		    arg++;
@@ -607,6 +645,7 @@ void Usage(void)
     fprintf(ErrFp, "Options:\n");
     fprintf(ErrFp, " -n     Output next occurrence of reminders in simple format\n");
     fprintf(ErrFp, " -r     Disable RUN directives\n");
+    fprintf(ErrFp, " -@[n,m] Colorize COLOR reminders\n");
     fprintf(ErrFp, " -c[a][n] Produce a calendar for n (default 1) months\n");
     fprintf(ErrFp, " -c[a]+[n] Produce a calendar for n (default 1) weeks\n");
     fprintf(ErrFp, " -w[n[,p[,s]]]  Specify width, padding and spacing of calendar\n");
