@@ -11,7 +11,7 @@
 /*                                                             */
 /*  This file is part of REMIND.                               */
 /*  This file is Copyright (C) 1993-1998 by Mikko Silvonen.    */
-/*  REMIND is Copyright (C) 1992-2018 by Dianne Skoll          */
+/*  REMIND is Copyright (C) 1992-2020 by Dianne Skoll          */
 /*                                                             */
 /***************************************************************/
 
@@ -302,7 +302,8 @@ EXTERN char *ErrMsg[] =
     "Can't convert between time zones",
     "No files matching *.rem",
     "String too long",
-    "Time specified twice"
+    "Time specified twice",
+    "Cannot specify DURATION without specifying AT"
 
 #elif IBMEXTENDED
     "Ok",
@@ -407,7 +408,8 @@ EXTERN char *ErrMsg[] =
     "Can't convert between time zones",
     "No files matching *.rem",
     "String too long",
-    "Time specified twice"
+    "Time specified twice",
+    "Cannot specify DURATION without specifying AT"
 
 #else
     "Ok",
@@ -512,7 +514,8 @@ EXTERN char *ErrMsg[] =
     "Can't convert between time zones",
     "No files matching *.rem",
     "String too long",
-    "Time specified twice"
+    "Time specified twice",
+    "Cannot specify DURATION without specifying AT"
 
 
 #endif
@@ -524,7 +527,7 @@ EXTERN char *ErrMsg[] =
 #define L_USAGE_OVERRIDE 1
 void Usage(void)
 {
-    fprintf(ErrFp, "\nREMIND %s (%s version) Copyright 1992-2018 Dianne Skoll\n", VERSION, L_LANGNAME);
+    fprintf(ErrFp, "\nREMIND %s (%s version) Copyright 1992-2020 Dianne Skoll\n", VERSION, L_LANGNAME);
 #ifdef BETA
     fprintf(ErrFp, ">>>> BETAVERSIO <<<<\n");
 #endif
