@@ -8,7 +8,7 @@
 /*                                                             */
 /*  This file is part of REMIND.                               */
 /*                                                             */
-/*  REMIND is Copyright (C) 1992-2018 by Dianne Skoll          */
+/*  REMIND is Copyright (C) 1992-2020 by Dianne Skoll          */
 /*  This file is Copyright (C) 1996 by Marco Paganini and      */
 /*  Dianne Skoll.                                              */
 /*                                                             */
@@ -247,7 +247,8 @@ EXTERN char *ErrMsg[] =
     "Can't convert between time zones",
     "No files matching *.rem",
     "String too long",
-    "Time specified twice"
+    "Time specified twice",
+    "Cannot specify DURATION without specifying AT"
 };
 #endif /* MK_GLOBALS */
 
@@ -256,7 +257,7 @@ EXTERN char *ErrMsg[] =
 #define L_USAGE_OVERRIDE 1
 void Usage(void)
 {
-    fprintf(ErrFp, "\nREMIND %s (versao %s) (C) 1992-2018 Dianne Skoll\n", VERSION, L_LANGNAME);
+    fprintf(ErrFp, "\nREMIND %s (versao %s) (C) 1992-2020 Dianne Skoll\n", VERSION, L_LANGNAME);
 #ifdef BETA
     fprintf(ErrFp, ">>>> VERSAO BETA <<<<\n");
 #endif
