@@ -127,11 +127,7 @@ int main(int argc, char *argv[])
 	    }
 	}
 	if (Iterations) {
-	    ClearGlobalOmits();
-	    DestroyOmitContexts();
-	    DestroyVars(0);
-	    NumTriggered = 0;
-	    ClearLastTriggers();
+	    PerIterationInit();
 	    JulianToday++;
 	}
     }
@@ -149,6 +145,19 @@ void PurgeEchoLine(char const *fmt, ...)
 
 }
 
+void
+PerIterationInit(void)
+{
+    ClearGlobalOmits();
+    DestroyOmitContexts();
+    DestroyVars(0);
+    DefaultColorR = -1;
+    DefaultColorG = -1;
+    DefaultColorB = -1;
+    NumTriggered = 0;
+    ClearLastTriggers();
+}
+
 /***************************************************************/
 /*                                                             */
 /*  DoReminders                                                */
