@@ -20,6 +20,7 @@
 #include <unistd.h>
 #include <stdlib.h>
 #include "rem2ps.h"
+#include "json.h"
 
 #define NEW(type) (malloc(sizeof(type)))
 
@@ -30,6 +31,7 @@
 #define SPECIAL_COLOR      4
 #define SPECIAL_WEEK       5
 #define SPECIAL_SHADE      6
+#define SPECIAL_UNKNOWN    7
 
 /* Array holding how specials sort */
 static int SpecialSortOrder[] = {
@@ -46,6 +48,7 @@ typedef struct calentry {
     struct calentry *next;
     int special;
     char *entry;
+    int daynum;
 } CalEntry;
 
 typedef struct {
@@ -137,6 +140,172 @@ void WriteOneEntry (CalEntry *c);
 void GetSmallLocations (void);
 char const *EatToken(char const *in, char *out, int maxlen);
 
+/***************************************************************/
+/*                                                             */
+/*  StrCmpi                                                    */
+/*                                                             */
+/*  Compare strings, case insensitive.                         */
+/*                                                             */
+/***************************************************************/
+int StrCmpi(char const *s1, char const *s2)
+{
+    int r;
+    while (*s1 && *s2) {
+	r = toupper(*s1) - toupper(*s2);
+	if (r) return r;
+	s1++;
+	s2++;
+    }
+    return toupper(*s1) - toupper(*s2);
+}
+/***************************************************************/
+/*                                                             */
+/*   Parse the new-style JSON intermediate format              */
+/*                                                             */
+/***************************************************************/
+static CalEntry *
+JSONToCalEntry(DynamicBuffer *buf)
+{
+    CalEntry *c;
+    json_value *val;
+
+    val = json_parse(DBufValue(buf), DBufLen(buf));
+    if (!val) {
+	fprintf(stderr, "Unable to parse JSON line `%s'\n", DBufValue(buf));
+	exit(1);
+    }
+
+    if (val->type != json_object) {
+	fprintf(stderr, "Expecting JSON object; found `%s'\n",
+		DBufValue(buf));
+	exit(1);
+    }
+
+    c = NEW(CalEntry);
+    if (!c) {
+	fprintf(stderr, "malloc failed - aborting.\n");
+	exit(1);
+    }
+    c->next = NULL;
+    c->special = SPECIAL_NORMAL;
+
+    int got_date = 0, got_body = 0;
+    size_t i;
+    for (i=0; i<val->u.object.length; i++) {
+	char const *nm = val->u.object.values[i].name;
+	json_value *v = val->u.object.values[i].value;
+	char const *s;
+	if (!strcmp(nm, "date")) {
+	    if (v->type == json_string) {
+		s = v->u.string.ptr;
+		c->daynum = (s[8] - '0') * 10 + s[9] - '0';
+		got_date = 1;
+	    }
+	} else if (!strcmp(nm, "body")) {
+	    if (v->type == json_string) {
+		s = v->u.string.ptr;
+		c->entry = malloc(strlen(s)+1);
+		if (!c->entry) {
+		    fprintf(stderr, "malloc failed - aborting.\n");
+		    exit(1);
+		}
+		strcpy(c->entry, s);
+		got_body = 1;
+	    }
+	} else if (!strcmp(nm, "passthru")) {
+	    if (v->type == json_string) {
+		s = v->u.string.ptr;
+		if (!StrCmpi(s, "PostScript")) {
+		    c->special = SPECIAL_POSTSCRIPT;
+		} else if (!StrCmpi(s, "SHADE")) {
+		    c->special = SPECIAL_SHADE;
+		} else if (!StrCmpi(s, "MOON")) {
+		    c->special = SPECIAL_MOON;
+		} else if (!StrCmpi(s, "WEEK")) {
+		    c->special = SPECIAL_WEEK;
+		} else if (!StrCmpi(s, "PSFile")) {
+		    c->special = SPECIAL_PSFILE;
+		} else if (!StrCmpi(s, "COLOUR") ||
+			   !StrCmpi(s, "COLOR")) {
+		    c->special = SPECIAL_COLOR;
+		} else {
+		    c->special = SPECIAL_UNKNOWN;
+		}
+	    }
+	}
+    }
+
+    json_value_free(val);
+
+    if (!got_body || !got_date) {
+	fprintf(stderr, "Could not parse line `%s'\n", DBufValue(buf));
+	exit(1);
+    }
+    return c;
+}
+
+/***************************************************************/
+/*                                                             */
+/*   Parse the old-style REM2PS intermediate format            */
+/*                                                             */
+/***************************************************************/
+static CalEntry *
+TextToCalEntry(DynamicBuffer *buf)
+{
+    char const *startOfBody;
+    char passthru[PASSTHRU_LEN+1];
+
+    CalEntry *c = NEW(CalEntry);
+    if (!c) {
+	fprintf(stderr, "malloc failed - aborting.\n");
+	exit(1);
+    }
+    c->next = NULL;
+    c->special = SPECIAL_NORMAL;
+    c->daynum = (DBufValue(buf)[8] - '0') * 10 + DBufValue(buf)[9] - '0';
+
+    /* Skip the tag, duration and time */
+    startOfBody = DBufValue(buf)+10;
+
+    /* Eat the passthru */
+    startOfBody = EatToken(startOfBody, passthru, PASSTHRU_LEN);
+
+    /* Eat the tag */
+    startOfBody = EatToken(startOfBody, NULL, 0);
+
+    /* Eat the duration */
+    startOfBody = EatToken(startOfBody, NULL, 0);
+
+    /* Eat the time */
+    startOfBody = EatToken(startOfBody, NULL, 0);
+
+    c->entry = malloc(strlen(startOfBody) + 1);
+    if (!c->entry) {
+	fprintf(stderr, "malloc failed - aborting.\n");
+	exit(1);
+    }
+    strcpy(c->entry, startOfBody);
+
+    /* Save the type of SPECIAL */
+    if (!StrCmpi(passthru, "PostScript")) {
+	c->special = SPECIAL_POSTSCRIPT;
+    } else if (!StrCmpi(passthru, "SHADE")) {
+	c->special = SPECIAL_SHADE;
+    } else if (!StrCmpi(passthru, "MOON")) {
+	c->special = SPECIAL_MOON;
+    } else if (!StrCmpi(passthru, "WEEK")) {
+	c->special = SPECIAL_WEEK;
+    } else if (!StrCmpi(passthru, "PSFile")) {
+	c->special = SPECIAL_PSFILE;
+    } else if (!StrCmpi(passthru, "COLOUR") ||
+	       !StrCmpi(passthru, "COLOR")) {
+	c->special = SPECIAL_COLOR;
+    } else if (StrCmpi(passthru, "*")) {
+	c->special = SPECIAL_UNKNOWN;
+    }
+    return c;
+}
+
 /***************************************************************/
 /*                                                             */
 /*   MAIN PROGRAM                                              */
@@ -154,10 +323,17 @@ int main(int argc, char *argv[])
 	Usage("Input should not come from a terminal");
     }
 
+    int first_line = 1;
     /* Search for a valid input file */
     while (!feof(stdin)) {
 	DBufGets(&buf, stdin);
-	if (!strcmp(DBufValue(&buf), PSBEGIN)) {
+	if (first_line && (!strcmp(DBufValue(&buf), "["))) {
+	    fprintf(stderr, "Rem2PS: It appears that you have invoked Remind with the -ppp option.\n        Please use either -p or -pp, but not -ppp.\n");
+	    exit(1);
+	}
+	first_line = 0;
+	if (!strcmp(DBufValue(&buf), PSBEGIN) ||
+	    !strcmp(DBufValue(&buf), PSBEGIN2)) {
 	    if (!validfile) {
 		if (Verbose) {
 		    fprintf(stderr, "Rem2PS: Version %s Copyright 1992-2020 by Dianne Skoll\n\n", VERSION);
@@ -191,10 +367,7 @@ void DoPsCal(void)
     int days, wkday, prevdays, nextdays;
     int sfirst;
     int i;
-    int is_ps;
     int firstcol;
-    char const *startOfBody;
-    char passthru[PASSTHRU_LEN+1];
     DynamicBuffer buf;
     CalEntry *c, *d, *p;
 
@@ -230,6 +403,7 @@ void DoPsCal(void)
 
     printf("%%%%Page: %c%c%c%c%c %d\n", month[0], month[1], month[2],
 	   year[2], year[3], validfile);
+    printf("%%%%PageBoundingBox: 0 0 %d %d\n", CurPage->xsize, CurPage->ysize);
 
 /* Emit PostScript to do the heading */
     if (!PortraitMode) printf("90 rotate 0 XSIZE neg translate\n");
@@ -272,80 +446,48 @@ void DoPsCal(void)
 	}
 
 	DBufGets(&buf, stdin);
-	if (!strcmp(DBufValue(&buf), PSEND)) {
+	if (!strcmp(DBufValue(&buf), PSEND) ||
+	    !strcmp(DBufValue(&buf), PSEND2)) {
 	    DBufFree(&buf);
 	    break;
 	}
 
 	/* Ignore lines beginning with '#' */
 	if (DBufValue(&buf)[0] == '#') {
+	    DBufFree(&buf);
 	    continue;
 	}
-	/* Read the day number - a bit of a hack! */
-	DayNum = (DBufValue(&buf)[8] - '0') * 10 + DBufValue(&buf)[9] - '0';
-	if (DayNum != CurDay) {
-	    for(; CurDay<DayNum; CurDay++) {
+
+	if (DBufValue(&buf)[0] == '{') {
+	    /* Starts with '{', so assume new-style JSON format */
+	    c = JSONToCalEntry(&buf);
+	} else {
+	    /* Assume it's the old-style rem2ps intermediate format */
+	    c = TextToCalEntry(&buf);
+	}
+
+	/* If it's an unknown special, ignore */
+	if (c->special == SPECIAL_UNKNOWN) {
+	    DBufFree(&buf);
+	    free(c);
+	    c = NULL;
+	    continue;
+	}
+	if (c->daynum != CurDay) {
+	    for(; CurDay<c->daynum; CurDay++) {
 		WriteCalEntry();
 		WkDayNum = (WkDayNum + 1) % 7;
 	    }
 	}
-	/* Add the text */
-	c = NEW(CalEntry);
-	if (!c) {
-	    fprintf(stderr, "malloc failed - aborting.\n");
-	    exit(1);
-	}
-	c->next = NULL;
-	c->special = SPECIAL_NORMAL;
-
-	/* Skip the tag, duration and time */
-	startOfBody = DBufValue(&buf)+10;
-
-	/* Eat the passthru */
-	startOfBody = EatToken(startOfBody, passthru, PASSTHRU_LEN);
-
-	/* Eat the tag */
-	startOfBody = EatToken(startOfBody, NULL, 0);
-
-	/* Eat the duration */
-	startOfBody = EatToken(startOfBody, NULL, 0);
-
-	/* Eat the time */
-	startOfBody = EatToken(startOfBody, NULL, 0);
-
-	is_ps = 0;
-	if (!strcmp(passthru, "PostScript") ||
-	    !strcmp(passthru, "PSFile") ||
-	    !strcmp(passthru, "MOON") ||
-	    !strcmp(passthru, "WEEK") ||
-	    !strcmp(passthru, "SHADE")) {
-	    is_ps = 1;
-	}
-	c->entry = malloc(strlen(startOfBody) + 1);
-	if (!c->entry) {
-	    fprintf(stderr, "malloc failed - aborting.\n");
-	    exit(1);
-	}
-	strcpy(c->entry, startOfBody);
-
-	if (is_ps) {
-	    /* Save the type of SPECIAL */
-	    if (!strcmp(passthru, "PostScript")) {
-		c->special = SPECIAL_POSTSCRIPT;
-	    } else if (!strcmp(passthru, "SHADE")) {
-		c->special = SPECIAL_SHADE;
-	    } else if (!strcmp(passthru, "MOON")) {
-		c->special = SPECIAL_MOON;
-	    } else if (!strcmp(passthru, "WEEK")) {
-		c->special = SPECIAL_WEEK;
+	if (c->special == SPECIAL_POSTSCRIPT ||
+	    c->special == SPECIAL_SHADE ||
+	    c->special == SPECIAL_MOON ||
+	    c->special == SPECIAL_WEEK ||
+	    c->special == SPECIAL_PSFILE) {
+	    if (!PsEntries[c->daynum]) {
+		PsEntries[c->daynum] = c;
 	    } else {
-		c->special = SPECIAL_PSFILE;
-	    }
-
-	    if (!PsEntries[DayNum]) {
-		PsEntries[DayNum] = c;
-	    } else {
-		d = PsEntries[DayNum];
+		d = PsEntries[c->daynum];
 		p = NULL;
 		/* Slot it into the right place */
 		while (d->next && (SpecialSortOrder[c->special] <= SpecialSortOrder[d->special])) {
@@ -359,14 +501,12 @@ void DoPsCal(void)
 		    if (p) {
 			p->next = c;
 		    } else {
-			PsEntries[DayNum] = c;
+			PsEntries[c->daynum] = c;
 		    }
 		    c->next = d;
 		}
 	    }
-	} else if (!strcmp(passthru, "*") ||
-		   !strcmp(passthru, "COLOUR") ||
-	           !strcmp(passthru, "COLOR")) {
+	} else {
 	    /* Put on linked list */
 	    if (!CurEntries) {
 		CurEntries = c;
@@ -375,10 +515,6 @@ void DoPsCal(void)
 		while(d->next) d = d->next;
 		d->next = c;
 	    }
-	    if (!strcmp(passthru, "COLOR") ||
-		!strcmp(passthru, "COLOUR")) {
-		c->special = SPECIAL_COLOR;
-	    }
 	}
     }
     for(; CurDay<=days; CurDay++) {
