@@ -5,7 +5,7 @@
 /*  Routines for figuring out the trigger date of a reminder   */
 /*                                                             */
 /*  This file is part of REMIND.                               */
-/*  Copyright (C) 1992-2018 by Dianne Skoll                    */
+/*  Copyright (C) 1992-2020 by Dianne Skoll                    */
 /*                                                             */
 /***************************************************************/
 
@@ -408,6 +408,61 @@ static int GetNextTriggerDate(Trigger *trig, int start, int *err, int *nextstart
     return simple;
 }
 
+int
+AdjustTriggerForDuration(int today, int r, Trigger *trig, TimeTrig *tim, int save_in_globals)
+{
+    int y, m, d;
+    /* If we have an AT, save the original event start */
+    if (tim->ttime != NO_TIME) {
+	trig->eventstart = MINUTES_PER_DAY * r + tim->ttime;
+	if (tim->duration != NO_TIME) {
+	    trig->eventduration = tim->duration;
+	}
+    }
+
+    /* Now potentially adjust */
+    if (r < today && r + trig->duration_days >= today) {
+	/* Adjust duration down */
+	tim->duration -= (today - r) * MINUTES_PER_DAY;
+	tim->duration += tim->ttime;
+
+	/* Start at midnight */
+	tim->ttime = 0;
+
+	/* Change trigger date to today */
+	r = today;
+	if (DebugFlag & DB_PRTTRIG) {
+	    FromJulian(r, &y, &m, &d);
+	    fprintf(ErrFp, "%s(%d): Trig(adj) = %s, %d %s, %d",
+		    FileName, LineNo,
+		    DayName[r % 7],
+		    d,
+		    MonthName[m],
+		    y);
+	    if (tim->ttime != NO_TIME) {
+		fprintf(ErrFp, " AT %02d:%02d",
+			(tim->ttime / 60),
+			(tim->ttime % 60));
+		if (tim->duration != NO_TIME) {
+		    fprintf(ErrFp, " DURATION %02d:%02d",
+			    (tim->duration / 60),
+			    (tim->duration % 60));
+		}
+	    }
+	    fprintf(ErrFp, "\n");
+	}
+
+    }
+    if (save_in_globals) {
+	LastTriggerTime = tim->ttime;
+	SaveLastTimeTrig(tim);
+	SaveLastTrigger(trig);
+	LastTriggerDate = r;
+	LastTrigValid = 1;
+    }
+    return r;
+}
+
 /***************************************************************/
 /*                                                             */
 /*  ComputeTrigger                                             */
@@ -416,10 +471,30 @@ static int GetNextTriggerDate(Trigger *trig, int start, int *err, int *nextstart
 /*  today's date.                                              */
 /*                                                             */
 /***************************************************************/
-int ComputeTrigger(int today, Trigger *trig, int *err, int save_in_globals)
+int ComputeTrigger(int today, Trigger *trig, TimeTrig *tim,
+		   int *err, int save_in_globals)
+{
+    int r = ComputeTriggerNoAdjustDuration(today, trig, tim, err, save_in_globals);
+    if (*err != OK) {
+	return r;
+    }
+    r = AdjustTriggerForDuration(today, r, trig, tim, save_in_globals);
+    return r;
+}
+
+/***************************************************************/
+/*                                                             */
+/*  ComputeTriggerNoAdjustDuration                             */
+/*                                                             */
+/*  Compute a trigger, but do NOT adjust the time trigger      */
+/*  duration.                                                  */
+/*                                                             */
+/***************************************************************/
+int ComputeTriggerNoAdjustDuration(int today, Trigger *trig, TimeTrig *tim,
+				   int *err, int save_in_globals)
 {
     int nattempts = 0,
-	start = today,
+	start = today - trig->duration_days,
 	nextstart = 0,
 	y, m, d, omit,
 	result;
@@ -429,15 +504,25 @@ int ComputeTrigger(int today, Trigger *trig, int *err, int save_in_globals)
 	LastTrigValid = 0;
     }
 
-/* Assume everything works */
+    /* Assume everything works */
     *err = OK;
 
-/* But check for obvious problems... */
+    /* But check for obvious problems... */
     if (trig->localomit == 1 + 2 + 4 + 8 + 16 + 32 + 64) {
 	*err = E_2MANY_LOCALOMIT;
 	return -1;
     }
 
+    if (start < 0) {
+	*err = E_DATE_OVER;
+	return -1;
+    }
+
+    if (tim->duration != NO_TIME && tim->ttime == NO_TIME) {
+	*err = E_DURATION_NO_AT;
+	return -1;
+    }
+
     if (trig->rep != NO_REP &&
 	(trig->d == NO_DAY ||
 	 trig->m == NO_MON ||
@@ -448,6 +533,11 @@ int ComputeTrigger(int today, Trigger *trig, int *err, int save_in_globals)
     }
 
 
+    /* Save the trigger */
+    if (save_in_globals) {
+	SaveLastTrigger(trig);
+    }
+
     while (nattempts++ < TRIG_ATTEMPTS) {
 	result = GetNextTriggerDate(trig, start, err, &nextstart);
 
@@ -469,7 +559,9 @@ int ComputeTrigger(int today, Trigger *trig, int *err, int save_in_globals)
 	} else {
 	    omit = 0;
 	}
-	if (result >= today &&
+
+	/** FIXME: Fix bad interaction with SATISFY... need to rethink!!! */
+	if (result+trig->duration_days >= today &&
 	    (trig->skip != SKIP_SKIP || !omit)) {
 	    if (save_in_globals) {
 		LastTriggerDate = result;  /* Save in global var */
@@ -477,12 +569,23 @@ int ComputeTrigger(int today, Trigger *trig, int *err, int save_in_globals)
 	    }
 	    if (DebugFlag & DB_PRTTRIG) {
 		FromJulian(result, &y, &m, &d);
-		fprintf(ErrFp, "%s(%d): Trig = %s, %d %s, %d\n",
+		fprintf(ErrFp, "%s(%d): Trig = %s, %d %s, %d",
 			FileName, LineNo,
 			DayName[result % 7],
 			d,
 			MonthName[m],
 			y);
+		if (tim->ttime != NO_TIME) {
+		    fprintf(ErrFp, " AT %02d:%02d",
+			    (tim->ttime / 60),
+			    (tim->ttime % 60));
+		    if (tim->duration != NO_TIME) {
+			fprintf(ErrFp, " DURATION %02d:%02d",
+			    (tim->duration / 60),
+			    (tim->duration % 60));
+		    }
+		}
+		fprintf(ErrFp, "\n");
 	    }
 	    return result;
 	}
@@ -535,3 +638,41 @@ int ComputeTrigger(int today, Trigger *trig, int *err, int save_in_globals)
     *err = E_CANT_TRIG;
     return -1;
 }
+
+/***************************************************************/
+/*                                                             */
+/*  ComputeScanStart                                            */
+/*                                                             */
+/*  Figure out where to start scan from by examining SCANFROM  */
+/*  and DURATION                                               */
+/*                                                             */
+/***************************************************************/
+int
+ComputeScanStart(int today, Trigger *trig, TimeTrig *tt)
+{
+    int minutes, days;
+
+    /* If we don't have a time/duration, just use scanfrom */
+    if (tt->ttime == NO_TIME ||
+	tt->duration == NO_TIME) {
+	if (trig->scanfrom == NO_DATE) {
+	    return today;
+	}
+	return trig->scanfrom;
+    }
+
+    /* Calculate time-based SCANFROM */
+    minutes = tt->ttime + tt->duration - 1;
+
+    /* Figure out how many days to scan backwards from */
+    days = minutes / MINUTES_PER_DAY;
+
+    if (trig->scanfrom != NO_DATE) {
+	if (trig->scanfrom <= today - days) {
+	    return trig->scanfrom;
+	} else {
+	    return today - days;
+	}
+    }
+    return today - days;
+}
