@@ -216,3 +216,18 @@ typedef struct {
 /* Flags for FROM / SCANFROM */
 #define SCANFROM_TYPE 0
 #define FROM_TYPE     1
+
+/* PS Calendar levels */
+
+/* Original interchange format */
+#define PSCAL_LEVEL1  1
+
+/* Line-by-line JSON */
+#define PSCAL_LEVEL2  2
+
+/* Pure JSON */
+#define PSCAL_LEVEL3  3
+
+#define TERMINAL_BACKGROUND_UNKNOWN 0
+#define TERMINAL_BACKGROUND_DARK    1
+#define TERMINAL_BACKGROUND_LIGHT   2
