@@ -168,6 +168,45 @@ static int datetime_sep_func(int do_set, Value *val)
     return OK;
 }
 
+static int default_color_func(int do_set, Value *val)
+{
+    int col_r, col_g, col_b;
+    if (!do_set) {
+    /* 12 = strlen("255 255 255\0") */
+ val->v.str = malloc(12);
+ if (!val->v.str) return E_NO_MEM;
+    snprintf(val->v.str, 12, "%d %d %d",
+        DefaultColorR,
+        DefaultColorG,
+        DefaultColorB
+        );
+ val->type = STR_TYPE;
+ return OK;
+    }
+    if (val->type != STR_TYPE) return E_BAD_TYPE;
+    if (sscanf(val->v.str, "%d %d %d", &col_r, &col_g, &col_b) != 3) {
+        return E_BAD_TYPE;
+    }
+    /* They either all have to be -1, or all between 0 and 255 */
+    if (col_r == -1 && col_g == -1 && col_b == -1) {
+	DefaultColorR = -1;
+	DefaultColorG = -1;
+	DefaultColorB = -1;
+	return OK;
+    }
+    if (col_r < 0) return E_2LOW;
+    if (col_r > 255) return E_2HIGH;
+    if (col_g < 0) return E_2LOW;
+    if (col_g > 255) return E_2HIGH;
+    if (col_b < 0) return E_2LOW;
+    if (col_b > 255) return E_2HIGH;
+
+    DefaultColorR = col_r;
+    DefaultColorG = col_g;
+    DefaultColorB = col_b;
+    return OK;
+}
+
 static int date_sep_func(int do_set, Value *val)
 {
     if (!do_set) {
@@ -608,6 +647,7 @@ static SysVar SysVarArr[] = {
     {"Daemon",         0,  INT_TYPE,     &Daemon,             0,      0   },
     {"DateSep",        1,  SPECIAL_TYPE, date_sep_func,       0,      0   },
     {"DateTimeSep",    1,  SPECIAL_TYPE, datetime_sep_func,   0,      0   },
+    {"DefaultColor",   1,  SPECIAL_TYPE, default_color_func,  0,      0   },
     {"DefaultPrio",    1,  INT_TYPE,     &DefaultPrio,        0,      9999},
     {"DeltaOffset",    0,  INT_TYPE,     &DeltaOffset,        0,      0   },
     {"DontFork",       0,  INT_TYPE,     &DontFork,           0,      0   },
